@@ -144,6 +144,7 @@ static inline void bit2_to_bytes(const char *bit2_ptr, const Py_ssize_t bit2_n,
 
   dna_i = 0;
   for (bit2_i=0; bit2_i < bit2_n; bit2_i++)
+    //printf("dna_i: %zd, bit2_i: %zd\n", dna_i, bit2_i);
     {
       cur_byte = bit2_ptr[bit2_i];
       for (bitpos = 6; bitpos >= 0; bitpos -= 2) {
@@ -362,6 +363,7 @@ static inline void make_sliceboundaries(const Py_ssize_t ilow,
 					const Py_ssize_t ihigh, 
 					SliceBoundaries *slbo)
 {
+  /* */
   slbo->low_n_padbytes = ((ilow % NGSP_WORD_SIZE) == 0) ? 0 : NGSP_WORD_SIZE - (ilow % NGSP_WORD_SIZE);
   slbo->low_offset = (slbo->low_n_padbytes == 0)?0:1;
   slbo->high_n_padbytes = ihigh % NGSP_WORD_SIZE;
@@ -382,11 +384,9 @@ static inline void packeddna_fromslice(const Py_ssize_t ilow,
 }
 
 
-PyDoc_STRVAR(slicebit2_frombit2bytes_doc,
-"Takes a bytes object with 2-bit packed DNA values, a slice in the DNA space,"
-" and returns a new bytes object with 2-bit packed DNA values."
-" As with other Python slices the last element is not included.");
 
+PyDoc_STRVAR(slicebit2_frombit2bytes_doc,
+"Takes a bytes object with 2-bit packed DNA values, a slice with coordinates on the unpacked sequence, and returns a 2-bit packed slice. As with other Python slices the last element is not included.");
 static PyObject* slicebit2_frombit2bytes(PyObject *self, PyObject *args)
 {
 
@@ -396,7 +396,6 @@ static PyObject* slicebit2_frombit2bytes(PyObject *self, PyObject *args)
 			&ilow, &ihigh)) { 
     return NULL; 
   }
-  unsigned char *packed_char;
 
   if (ilow < 0) {
     PyErr_Format(PyExc_IndexError, 
@@ -405,56 +404,45 @@ static PyObject* slicebit2_frombit2bytes(PyObject *self, PyObject *args)
   }
   if (ihigh < ilow) {
     PyErr_Format(PyExc_IndexError, 
-		 "Lowest index must be lower that the highest.");
+		 "Lowest index must be lower than the highest.");
     return NULL;
   }
 
-  Py_ssize_t n = PyBytes_GET_SIZE(packed_bytes);
-  /* Offset if boundaries of the slice not multiple of NGSP_WORD_SIZE */
-  SliceBoundaries slbo;
-  make_sliceboundaries(ilow, ihigh, &slbo);
+  const Py_ssize_t n_p = PyBytes_GET_SIZE(packed_bytes);
+  unsigned char *packed = (unsigned char*) PyBytes_AS_STRING(packed_bytes);
 
-  if (slbo.ihigh_packed > n) {
-    PyErr_Format(PyExc_IndexError, 
+  const Py_ssize_t ilow_p = ilow / NGSP_WORD_SIZE;
+  const Py_ssize_t ihigh_p = (ihigh / NGSP_WORD_SIZE) + ((ihigh % 4 == 0) ? 0 : 1);
+  //FIXME: test >0
+  const int shiftlow = ilow % NGSP_WORD_SIZE;
+  const int shifthigh = NGSP_WORD_SIZE - shiftlow;
+  //printf("shiftlow: %i\n", shiftlow);
+
+  if (ihigh_p > n_p) {
+    PyErr_Format(PyExc_IndexError,
 		 "Highest limit for the interval out of bound "
-		 "(requesting %zd while length is %zd)", slbo.ihigh_packed, n);
+		 "(requesting %zd while length is %zd)", ihigh_p, n_p);
     return NULL;
   }
 
-  /*
-   * |----|-- --|- ---| ----|
-   *    ^----|----|----|-^
-   */
-
-  Py_ssize_t n_slice = (ihigh - ilow) / NGSP_WORD_SIZE;
-  if (((ihigh - ilow) % NGSP_WORD_SIZE) > 0)
-    n_slice++;
+  //const Py_ssize_t n_slicepacked = ihigh_p - ilow_p; 
+  const Py_ssize_t padding_size = (ihigh - ilow) % NGSP_WORD_SIZE;
+  const Py_ssize_t n_slicepacked = ((ihigh - ilow) / NGSP_WORD_SIZE) + \
+    (padding_size==0 ? 0 : 1); 
   char *string = NULL;
-  PyObject *slice_bytes = PyBytes_FromStringAndSize(string, n_slice);
-  unsigned char *slice_char = PyBytes_AS_STRING(slice_bytes);
-  packed_char = PyBytes_AS_STRING(packed_bytes);
-  Py_ssize_t i;
-  for (i = 0; i < n_slice; i++) {
-    slice_char[i] = (char)0; 
+  PyObject *slice_bytes = PyBytes_FromStringAndSize(string, n_slicepacked);
+  unsigned char *slicepacked = (unsigned char*) PyBytes_AS_STRING(slice_bytes);
+  Py_ssize_t i_res;
+  for (i_res = 0; i_res < n_slicepacked; i_res++) {
+    slicepacked[i_res] = (packed[ilow_p + i_res] << (shiftlow * 2));
+    if ((shiftlow != 0) && (i_res < (n_slicepacked + 1)) && (ilow_p + i_res + 1 < n_p)) {
+      slicepacked[i_res] |= (packed[ilow_p + i_res + 1] >> (shifthigh*2));
+    }
   }
-  unsigned short remainder_src, remainder_dest, shift_src, shift_dest;
-  Py_ssize_t i_src, i_dest;
-  unsigned char src;
-  /* printf("%i\n", packed_char[i / NGSP_WORD_SIZE]); */
-  for (i = ilow; i < ihigh; i++) {
-    /* printf("%i\n", i); */
-    remainder_src = i % NGSP_WORD_SIZE;
-    remainder_dest = (i-ilow) % NGSP_WORD_SIZE;
-    i_src = i / NGSP_WORD_SIZE;
-    i_dest = (i-ilow) / NGSP_WORD_SIZE;
-    shift_src = (NGSP_WORD_SIZE - remainder_src - 1) * 2;
-    shift_dest = (NGSP_WORD_SIZE - remainder_dest - 1) * 2;
-    src = (unsigned char)((packed_char[i_src] & (0x03 << shift_src)) >> shift_src);
-    slice_char[i_dest] |= (src << shift_dest);
-    /* printf("src: i=%i - reste=%i - shift=%i - value=%i\n", */
-    /* 	   i_src, remainder_src, shift_src, src); */
-    /* printf("dest: i=%i - reste=%i - shift=%i - new=%i - value=%i\n", */
-    /* 	   i_dest, remainder_dest, shift_dest, (src << shift_dest), slice_char[i_dest]); */
+  /* by convention pad with trailing 'A's the remaining bases */
+  if (padding_size != 0) {
+    const Py_ssize_t right_padding = NGSP_WORD_SIZE - padding_size;
+    slicepacked[n_slicepacked-1] &= (0xff << (right_padding * 2));
   }
   return slice_bytes;
 }
@@ -480,121 +468,57 @@ static PyObject* slice_frombit2bytes(PyObject *self, PyObject *args)
   }
   if (ihigh < ilow) {
     PyErr_Format(PyExc_IndexError, 
-		 "Lowest index must be, well, lower that the highest.");
+		 "Lowest index must be lower than the highest.");
     return NULL;
   }
 
-  Py_ssize_t n = PyBytes_GET_SIZE(packed_bytes);
-  /* if (ilow == Py_None) */
-  /*   ilow = 0; */
-  /* if (ihigh == Py_None) */
-  /*   ihigh = n * NGSP_WORD_SIZE; */
+  const Py_ssize_t n_p = PyBytes_GET_SIZE(packed_bytes);
+  unsigned char *packed = (unsigned char*) PyBytes_AS_STRING(packed_bytes);
 
-  /* Offset if boundaries of the slice not multiple of NGSP_WORD_SIZE */
-  SliceBoundaries slbo;
+  const Py_ssize_t ilow_p = ilow / NGSP_WORD_SIZE;
+  const Py_ssize_t ihigh_p = (ihigh / NGSP_WORD_SIZE) + ((ihigh % 4 == 0) ? 0 : 1);
+  const int shiftlow = ilow % NGSP_WORD_SIZE;
+  const int shifthigh = NGSP_WORD_SIZE - shiftlow;
 
-  make_sliceboundaries(ilow, ihigh, &slbo);
-
-  int k;
-
-  if (slbo.ihigh_packed > n) {
-    PyErr_Format(PyExc_IndexError, 
+  if (ihigh_p > (n_p+1)) {
+    PyErr_Format(PyExc_IndexError,
 		 "Highest limit for the interval out of bound "
-		 "(requesting %zd while length is %zd)", slbo.ihigh_packed, n);
+		 "(requesting %zd while length is %zd)", ihigh_p, n_p);
     return NULL;
   }
 
-  Py_ssize_t n_up = ihigh - ilow;
+  const int lpadding_size = (ilow % NGSP_WORD_SIZE) == 0 ? 0 : (NGSP_WORD_SIZE - ilow % NGSP_WORD_SIZE);
+  const int rpadding_size = ihigh % NGSP_WORD_SIZE;
+  const Py_ssize_t n_sliceunpacked = ihigh - ilow; 
   char *string = NULL;
-  PyObject *buf_bytes = PyBytes_FromStringAndSize(string, n_up);
-  char *unpacked = PyBytes_AS_STRING(buf_bytes);
+  PyObject *slice_bytes = PyBytes_FromStringAndSize(string, n_sliceunpacked);
+  char *sliceunpacked = PyBytes_AS_STRING(slice_bytes);
 
-#ifdef LIBDNA_DEBUG
-  printf("-");
-  for (k = 0; k < n_up; k++) {
-    unpacked[k] = 'N';
+  Py_ssize_t i_res;
+  int bitpos;
+  for (i_res = 0; i_res < lpadding_size; i_res++) {
+    bitpos = i_res * 2;
+    sliceunpacked[lpadding_size - i_res - 1] = \
+      _ACGT[(packed[ilow_p] >> bitpos) & 0x3];
   }
-  printf("-\n");
-#endif
+  
+  const Py_ssize_t ilow_offset = (lpadding_size != 0) ? 1 : 0;
+  const Py_ssize_t ihigh_offset = (rpadding_size != 0) ? 1 : 0;
+  bit2_to_bytes((char *)&packed[ilow_p + ilow_offset], 
+		ihigh_p - ihigh_offset - ilow_p - ilow_offset, 
+  		&sliceunpacked[i_res], ihigh - ilow - lpadding_size - rpadding_size);
 
-  char *packed = PyBytes_AS_STRING(packed_bytes);
-#ifdef LIBDNA_DEBUG
-  printf("-->(%zd, %zd) low: offest %i, pad: %zd, ilow_packed:%zd\n", 
-	 ilow, ihigh, slbo.low_offset, slbo.low_n_padbytes, 
-	 slbo.ilow_packed, slbo.ihigh_packed);
-  printf("-->(%zd, %zd) high: offest %i, pad: %zd, ihigh_packed:%zd\n", 
-	 ilow, ihigh, slbo.high_offset, slbo.high_n_padbytes, 
-	 slbo.ihigh_packed);
-#endif
-  if ( slbo.ilow_packed + slbo.low_offset < slbo.ihigh_packed ) {
-#if defined(LIBDNA_DEBUG)
-    printf(" packed[%zd], length = %zd\n", 
-	   slbo.ilow_packed + slbo.low_offset, 
-	   (slbo.ihigh_packed - slbo.ilow_packed - slbo.low_offset - slbo.high_offset));
-#endif
-    bit2_to_bytes(&(packed[slbo.ilow_packed + slbo.low_offset]),
-		  (slbo.ihigh_packed - slbo.ilow_packed - slbo.low_offset - slbo.high_offset),
-		  &(unpacked[slbo.low_n_padbytes]), 
-		  ihigh - ilow);
-  }
-  char padbytes[NGSP_WORD_SIZE];
-#if defined(LIBDNA_DEBUG)
-  printf("-");
-  for (k = 0; k < n_up; k++) {
-    printf("%c", unpacked[k]);
-  }
-  printf("-\n");
-#endif
-
-  /* if offset on lower end, now take care of sub word-sized segment */
-  if (slbo.low_offset) {
-    //int foo = ((ihigh_packed - ilow_packed) > 1)?1:0;
-    int foo = 0;
-    bit2_to_bytes(&(packed[slbo.ilow_packed - foo]), 1,
-		  (char *)(&padbytes), NGSP_WORD_SIZE);
-    for (k = 0; k < slbo.low_n_padbytes; k++) {
-#ifdef LIBDNA_DEBUG
-      printf("  --> %i : padbytes[%i] = %c\n", k, 
-	     NGSP_WORD_SIZE - slbo.low_n_padbytes + k, 
-	     padbytes[NGSP_WORD_SIZE - slbo.low_n_padbytes + k]);//;k + NGSP_WORD_SIZE - low_n_padbytes);
-#endif
-      unpacked[k] = padbytes[NGSP_WORD_SIZE - slbo.low_n_padbytes + k];
-    }
-#ifdef LIBDNA_DEBUG
-  printf("post-low-fix\n-");
-  for (k = 0; k < n_up; k++) {
-    printf("%c", unpacked[k]);
-  }
-  printf("-\n");
-#endif
+  for (i_res = 0; i_res < rpadding_size; i_res++) {
+    bitpos = 6 - i_res * 2;
+    sliceunpacked[i_res + ihigh - ilow - rpadding_size] = \
+      _ACGT[(packed[ihigh_p-1] >> bitpos) & 0x3];
   }
 
-  /* if offset on higher end, now take care of sub word-sized segment */
-  if (slbo.high_offset) {
-    bit2_to_bytes(&(packed[slbo.ihigh_packed-1]), 1, // 10/3
-		  (char *)(&padbytes), NGSP_WORD_SIZE);
-    for (k = 0; k < slbo.high_n_padbytes; k++) {
-#ifdef LIBDNA_DEBUG
-      printf("  --> %i : padbytes[%i] = %c -> %zd\n", k, 
-	     k, 
-	     padbytes[k], ihigh - ilow - slbo.high_n_padbytes + k);//;k + NGSP_WORD_SIZE - low_n_padbytes);
-#endif
-      unpacked[ihigh - ilow - slbo.high_n_padbytes + k] = padbytes[k];
-    }
-
-#ifdef LIBDNA_DEBUG
-  printf("post-high-fix\n-");
-  for (k = 0; k < n_up; k++) {
-    printf("%c", unpacked[k]);
-  }
-  printf("-\n");
-#endif
-
-  }
-  return buf_bytes;
+  return slice_bytes;
 }
 
 
+
 PyDoc_STRVAR(slice_frombit2bytearray_doc,
 "Takes a bytearray object with 2-bit packed DNA values, a slice, and returns a 2-bit packed slice based on the unpacked indices. As with other Python slices the last element is not included.");
 static PyObject* slice_frombit2bytearray(PyObject *self, PyObject *args)
