@@ -106,6 +106,26 @@ def read_fasta(stream):
         raise FastaError('Missing FASTA header.')
     yield Entry(header, sequence)
 
+def write_fasta(buf, entry, seq_proc = None):
+    if entry.header != '>':
+        buf.write('> ')
+    buf.write(entry.header)
+    buf.write('\n')
+    if seq_proc is None:
+        buf.write(entry.sequence)
+    else:
+        buf.write(seq_proc(entry.sequence))
+    buf.write('\n')
+
+def write_qual(buf, entry, qual_proc = None):
+    buf.write('> ')
+    buf.write(entry.header)
+    buf.write('\n')
+    if qual_proc is None:
+        buf.write(entry.quality)
+    else:
+        buf.write(qual_proc(entry.sequence))
+
 EntryBin = namedtuple('Entry', 'header sequence bits')
 
 def tofastab(fasta, stream, force_upper = True):
