@@ -17,7 +17,7 @@
 
 from collections import namedtuple, OrderedDict
 
-import io, array
+import io, array, math
 import os, re
 import sys
 import h5py
@@ -66,11 +66,13 @@ class XSQFile(h5py.File):
     """
 
     @staticmethod
-    def _valid_library_name(name):
-        invalid_names = ('RunMetadata', 'Indexing', 'Unclassified')
+    def _valid_library_name(name, unclassified = False, unassigned = False):
+        invalid_names = ('RunMetadata', 'Indexing')
         if name in invalid_names:
             return False
-        if name.startswith('Unassigned'):
+        if not unclassified and name == 'Unclassified':
+            return False
+        if not unassigned and name.startswith('Unassigned'):
             return False
         return True
 
@@ -79,10 +81,11 @@ class XSQFile(h5py.File):
         
         return tuple(x for x in self.keys() if XSQFile._valid_library_name(x))
     library_names = property(_library_names_get, None, None,
-                             """ Names of the libraries in the file """)
+                             """ Names of the libraries in the file (Ignoring the unclassifed and unassigned groups)""")
 
-    def library(self, name):
-        if not XSQFile._valid_library_name(name):
+    def library(self, name, unclassified = False, unassigned = False):
+        if not XSQFile._valid_library_name(name, unclassified = unclassified,
+                                           unassigned = unassigned):
             raise ValueError("Invalid library name.")
         return XSQLibrary(self[name].id)
 
@@ -101,8 +104,6 @@ class XSQFile(h5py.File):
                             """ Metadata about the run """)
 
 
-
-
 class XSQLibrary(h5py._hl.group.Group):
     """
     Sequenced library in an XSQ file.
@@ -381,7 +382,7 @@ def seqid_template_csfasta(flowcell, fragment):
 
 def _progress_bar_iter(iterator, size = 20, out = sys.stdout, suffix = ''):
     n = len(iterator)
-    ms = n / size
+    ms = long(math.ceil(1.0 * n / size))
     for j, i in enumerate(iterator):
         if j % ms == 0:
             if os.isatty(out.fileno()):
@@ -506,7 +507,8 @@ def make_csfastq(lib, fragment, buf, f, flowcell,
 
 def automagic_fastq(filename, path_out = '.', fragment = None, 
                     buf_size = long(2E6), extension = 'fq', out = sys.stdout,
-                    cs = False):
+                    cs = False,
+                    want_unassigned = False):
     """ Extract sequence-space data into a FASTAQ file """
 
     if cs:
@@ -517,8 +519,13 @@ def automagic_fastq(filename, path_out = '.', fragment = None,
     f = XSQFile(filename, 'r')
     p = re.compile('.+_([0-9]+)\.xsq')
 
-    for i, lib_name in enumerate(f.library_names):
-        out.write('%s (%i / %i)\n' %(lib_name, i+1, len(f.library_names)))
+    if want_unassigned:
+        library_names = tuple(x for x in f.keys() if f._valid_library_name(x, unassigned = want_unassigned))
+    else:
+        library_names = f.library_names
+
+    for i, lib_name in enumerate(library_names):
+        out.write('%s (%i / %i)\n' %(lib_name, i+1, len(library_names)))
         out.flush()
         lib = f.library(lib_name)
         if fragment is None:
@@ -540,7 +547,8 @@ def automagic_fastq(filename, path_out = '.', fragment = None,
     f.close()
 
 def automagic_csfasta(filename, path_out = '.', fragment = None, 
-                      buf_size = long(2E6), out = sys.stdout):
+                      buf_size = long(2E6), out = sys.stdout,
+                      want_unassigned = False):
     """ Extract sequence-space data into a FASTAQ file """
 
     assert((fragment is None) or (fragment in FRAGMENT_NAMES))
@@ -548,11 +556,15 @@ def automagic_csfasta(filename, path_out = '.', fragment = None,
     f = XSQFile(filename, 'r')
     p = re.compile('.+_([0-9]+)\.xsq')
 
-
-    for i, lib_name in enumerate(f.library_names):
-        out.write('%s (%i / %i)\n' %(lib_name, i+1, len(f.library_names)))
+    if want_unassigned:
+        library_names = tuple(x for x in f.keys() if f._valid_library_name(x, unassigned = want_unassigned))
+    else:
+        library_names = f.library_names
+    
+    for i, lib_name in enumerate(library_names):
+        out.write('%s (%i / %i)\n' %(lib_name, i+1, len(library_names)))
         out.flush()
-        lib = f.library(lib_name)
+        lib = f.library(lib_name, unassigned = want_unassigned)
         if fragment is None:
             fragment_list = tuple(lib.fragments())
         else:
